@@ -23,6 +23,11 @@ try:
 except ImportError:
     import importlib_resources
 
+try:
+    import importlib.metadata as importlib_metadata
+except ImportError:
+    import importlib_metadata
+
 from pysrpm.convert import specifier_to_rpm_version, simplify_marker_to_rpm_condition
 
 
@@ -78,7 +83,8 @@ class RPM:
         self.tempdir = pathlib.Path(tempfile.mkdtemp())
         with tarfile.open(self.source) as tf:
             for name in tf.getnames():
-                if any(fnmatch.fnmatch(name, pat) for pat in ['*/pyproject.toml', '*/PKG-INFO', '*/setup.cfg']):
+                if any(fnmatch.fnmatch(name, pat) for pat in ['*/pyproject.toml', '*/PKG-INFO', '*/setup.cfg',
+                                                              '*/setup.py', '*/entry_points.txt']):
                     tf.extract(name, self.tempdir)
         try:
             self.root = next(self.tempdir.glob('*/PKG-INFO')).parent
@@ -222,39 +228,71 @@ class RPM:
 
         if with_deps or not pkg_info.exists():
             self.full_extraction()  # May be needed?
-            dist_meta = pep517.meta.load(root).metadata
+            dist = pep517.meta.load(root)
+            dist_meta = dist.metadata
+            entry_points = dist.entry_points
         else:
             # Metadata from importlib.metadata is an email.Message, so do the same here
             with open(pkg_info, 'rb') as f:
                 dist_meta = email.message_from_binary_file(f)
             dist_meta.set_param('charset', 'utf8')
 
+            # May not exist, only when egg-info is built (wheels, setuptools develop installs…)
+            ep_parser = configparser.ConfigParser()
+            try:
+                ep_parser.read_file([str(root / 'entry_points.txt')])
+            except configparser.MissingSectionHeaderError:
+                pass
+            entry_points = [importlib_metadata.EntryPoint(key, value, sec) for sec in ep_parser.sections()
+                            for key, value in parser.items(sec)]
+
         pyproject = root / 'pyproject.toml'
         self.build_system = {
             'requires': ['setuptools>=40.8.0', 'wheel'],
             'build-backend': 'setuptools.build_meta:__legacy__',
         }
-        if pyproject.exists():
-            with open(pyproject, 'rb') as f:
-                self.build_system.update(tomli.load(f)['build-system'])
-
         metadata = {
             'build-requires': self.build_system['requires'],
-            'build-backend': self.build_system['build-backend'],
             'long-description': dist_meta.get_payload().replace('%', '%%'),
         }
 
-        # See https://packaging.python.org/specifications/core-metadata/
-        multiple_use = {'Dynamic', 'Platform', 'Supported-Platform', 'Classifier', 'Requires-Dist',
-                        'Requires-External', 'Project-URL', 'Provides-Extra', 'Provides-Dist', 'Obsoletes-Dist'}
+        enabled_entry_points = []
+        disabled_entry_points = []
+        extra_patterns = self.templates['requires_extras'].split() + self.templates['suggests_extras'].split()
+        for ep in entry_points:
+            extras = re.match(r'.* \[(.+)\]$', ep.value.strip())
+            for extra in ((extra.strip() for extra in extras.group(1).split(',')) if extras else []):
+                if not any(fnmatch.fnmatch(extra, pattern) for pattern in extra_patterns):
+                    # A specified extra on the entry point is not matching any of the extras included in the package
+                    disabled_entry_points.append(f'"{ep.name}"' if ' ' in ep.name else ep.name)
+                    break
+            else:
+                enabled_entry_points.append(f'"{ep.name}"' if ' ' in ep.name else ep.name)
 
-        for key, value in dist_meta.items():
-            if key == 'Content-Type':
+        if enabled_entry_points:
+            metadata['entry-points'] = ' '.join(enabled_entry_points)
+        if disabled_entry_points:
+            metadata['disabled-entry-points'] = ' '.join(disabled_entry_points)
+
+        if pyproject.exists():
+            with open(pyproject, 'rb') as f:
+                self.build_system.update(tomli.load(f)['build-system'])
+                metadata['build-requires'] = self.build_system.get('build-requires', metadata['build-requires'])
+        # If running setup.py, we don’t need setuptools to be >= 40.8.0 as it is required for pyproject.toml:
+        elif (root / 'setup.py').exists():
+            metadata['build-requires'] = ['setuptools', 'wheel']
+
+        # See https://packaging.python.org/specifications/core-metadata/
+        multiple_use = {'dynamic', 'platform', 'supported-platform', 'classifier', 'requires-dist',
+                        'requires-external', 'project-url', 'provides-extra', 'provides-dist', 'obsoletes-dist'}
+
+        for key, value in ((key.lower(), value.replace('%', '%%')) for key, value in dist_meta.items()):
+            if key == 'content-type':
                 continue
             elif key in multiple_use:
-                metadata.setdefault(key.lower(), []).append(value.replace('%', '%%'))
+                metadata.setdefault(key, []).append(value)
             else:
-                metadata[key.lower()] = value.replace('%', '%%')
+                metadata[key] = f'{metadata[key]} {value}' if key in metadata else value
 
         return {
             **metadata,
